% Note: this is a sample script, it will not run without the fastaq files
%
% FASTQ file can be downloaded here: 
% http://www.chem.ualberta.ca/~derda/zebrapaper/    
% File name: Derda-sample2_NoIndex_L002_R1_001.fastq 
% File Size: 630 Mb
% This file will contain most sequences discussed in the manuscript
% note that abundances and copy number will be different because this
% file represents only 3.7% of the overall data.
%
% processing this file takes 8-15 minutes on an average desktop 
% 8 min on: iMac, OS X 10.6.8, 2.6 GHz Intel Core i5, 16 Gb 1333 MHz DDR3


name = {'Derda-sample2_NoIndex_L002_R1_001.fastq'};

%%%%    Illumina sequencing contained 27 FASTQ files (>10 Gb of data) 
%%%%    Unquoting this section will allow processing all files
%%%%    Files are available upon request.
%%%%    Processing 10 Gb data set takes ~ 2 hours.
%     for i=2:27
%         istr =  num2str(i);
%         name = [name; [name{1}(1:end-9) '0' ...
%                 char(48*ones(1,2-numel(istr))) istr '.fastq'] ];
%     end
    
% this is the name of the directory where FASTQ files are saved
indir = 'rawfiles';


% names of the directories where intermediate data is stored
outdirRAW = fullfile(indir,'RAW');
outdirPAR = fullfile(indir,'PAR');
outdirLIB = fullfile(indir,'LIB');
rawname   = 'data';
checkNNK = false;


% forward and reverse adapter seqeunces flanking the variable region
chunk=500000;
sF = 'TATTCTCACTCT';
sR = 'CGAACCTCCACC';

BAD= '#'; % Indicator of bad read in Phred+33
%BAD= 'B'; % Indicator of bad read in Phred+64

BARL = 3;  % length of the barcode
NNKL = 4;  % length of NKKNKK start

% %---------- define library markers 
marker = [];

marker(1).name = 'C7C';       % name of the marker

marker(1).FSseq = 'GCTTGT';   % sequence of the start marker in F-strand
marker(1).FSloc = [1 6];      % location of the start maker in F-strand
marker(1).RSseq = 'GCA';      % sequence of the start marker in R-strand
marker(1).RSloc = [1 3];      % location of the start maker in R-strand

marker(1).FEseq = 'TGCGGT';   % sequence of the start marker in F-strand
marker(1).FEloc = [28 33];      % location of the start maker in F-strand
marker(1).REseq = 'ACAAGCAGA';  % sequence of the start marker in R-strand
marker(1).REloc = [25 33];      % location of the start maker in R-strand

%-------------

marker(2).name = 'Ph7';       % name of the marker

marker(2).FSseq = '';   % sequence of the start marker in F-strand
marker(2).FSloc = [];      % location of the start maker in F-strand
marker(2).RSseq = '';      % sequence of the start marker in R-strand
marker(2).RSloc = [];      % location of the start maker in R-strand

marker(2).FEseq = 'GGTGGAGGTTCG';   % sequence of the start marker in F
marker(2).FEloc = [22 33];      % location of the start maker in F-strand
marker(2).REseq = 'AGAGTGAGAATA';  % sequence of the start marker in R
marker(2).REloc = [22 33];      % location of the start maker in R-strand
%-------------------------------------------------------------------------

%%% This part parses the FASTQ file and extracts raw sequences
Start = clock
disp(['Start Parsing: ' num2str(clock, '%10.0f') ]);
rawseq('inname', name, 'indir', indir,'outname',rawname, ...
       'outdir',outdirRAW,'chunk',chunk, 'searchF', sF, 'searchR', sR,...
       'BARL',BARL,'NNKL',NNKL,'marker',marker);
rawsTIME1 = clock - Start
disp(['************ rewseq TIME =' num2str(rawsTIME1,2) '********************'])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% cell array of strings defining different bar codes
BARS = {'GTA';'GAC'; 'CGA'; 'TTG'; 'TCA'};     


% names of the libraries and limit of the reads for difference libraries
LIBS = {'C7C';...
        'Ph7';...
        'P12'};
LIM   = {[7 27] [4 24];...
         [1 21] [1 21];...
         [1 33] [1 33]};
     
    


Quality = 30;   % All reads with Phred score below 30 will be discarded
CUTOFF = char(33+Quality); % cutoff for quality in Phred+33. 

% sort data by libraries and barcodes and discard low quality reads
sortlib2('rawname','parseq', 'indir', outdirRAW,'marker',marker,...
       'outdir',outdirLIB, 'CUTOFF', CUTOFF, 'outname', rawname,...
          'barcodes', BARS, 'libraries', LIBS, 'limits', LIM);

%name of the data file
uniname = ['unique' num2str(Quality)];

for i=1:numel(LIBS)  
    for j=1:numel(BARS)
        
        % name of the folder in which each library/barcoded data is stored
        FOL{i,j} = fullfile(outdirLIB, ['data122711' LIBS{i} '_' BARS{j}]);

        % generate unique sequences from the forward read
        uniseq('inname',fullfile(FOL{i,j},[rawname 'F.txt']), ...
               'outname', fullfile(FOL{i,j},[uniname 'F.txt']));

        % generate unique sequences from the reverse read   
        uniseq('inname',fullfile(FOL{i,j},[rawname 'R.txt']), ...
               'outname', fullfile(FOL{i,j},[uniname 'R.txt']));

        % find union of forward and reverse multisets  
        unionFR('inname1',fullfile(FOL{i,j},[uniname 'F.txt']), ...
                'inname2',fullfile(FOL{i,j},[uniname 'R.txt']), ...
                'outname', fullfile(FOL{i,j},[uniname 'FuR.txt']));
    end
end







