function look4mut(varargin)
% The script finds and tags point mutatins of the N most abundant sequences
% For N>10,0000, it is recommended to break the processing into chunks
% N can be  a million or more, as long as each processing chunk is
% ~10,000 sequences or less. The time is 1-3 minutes per chunk.

% the scripts can work with the file containing three colums:
% 1. nucleotide 2. peptide  3. frequency
% and converts it to a file with extra 4th column containing the tag
% Example: S000001.mut108. 
% S000001 - means this is the first seqeunce on the list
% mut108. - means that 108 mutants were found for this seq. in this file
%
% If seqeunce is a mutan of other seqeunce it will be tagged differently:
% S459556.mut0.S008118:C23T. 
% This tag means that sequence S459556 is a point mutant of S008118m
% the substitution occured in position 23. It's a C to T mutation
% mut0 means that S459556 has not point mutants. Note that the script 
% count mtants only once. a lower abundance sequence is a point mutant of a
% higher abundance seqeunce but not vice versa.
%
% If sequence is a point mutant of multiple sequences, it will have mltiple
% tags following one another and separated by "."
% Example: S459546.mut0.S003524:A7G.S017952:T4G.S246086:C35G.


inname = '';   
indir  = '';
outname= 'mutants';   % default save name
outdir = '';
TOTAL = 500000;         % total number of sequences to analyze
chunk = 5000;            % how manu chunks to break analysis into

%check for input variable
if exist('varargin','var')
    L = length(varargin);
    if rem(L,2) ~= 0, error('Parameters/Values must come in pairs.'); 
    end

    % read input variables
    for ni = 1:2:L
        switch lower(varargin{ni})
            case 'inname', inname = varargin{ni+1};
            case 'indir', indir=varargin{ni+1};
            case 'outdir', outdir = varargin{ni+1};
            case 'outname', outname = varargin{ni+1};
            case 'chunk', chunk = varargin{ni+1};
        end
    end
end

%------processing section------------------------------
    
tic    
A1 = fullfile(indir, inname);
disp(['reading ' A1 ]);
Open = fopen(A1, 'r+');
AllVar = textscan(Open,'%s %*s %f'); 

Nuc = AllVar{1}; % lists all nucleotides from column 1
Freq= AllVar{2}; % lists all frequencies from columns 3
clear AllVar;
fclose('all');

%--------- remove the nucleotide strings which are all blanks

ix=cellfun('isempty', Nuc);
Remain = find(ix==0);
Nuc = Nuc(Remain);
Freq = Freq(Remain);
N=numel(Nuc);

%--------  remove nucleotide strings with at least one blank
cNuc=char(Nuc);
[row,~] = find(cNuc==' ');
ALL     = 1:1:N;
IX      = setdiff(ALL,row);
Nuc  = Nuc(IX);
Freq = Freq(IX);
clear('IX','row','ALL');

%-------------------- recalculate unique frequencies -----------
[Nuc, Freq, ~] = uniqueCOMB (Nuc, Freq); 


% for analysis define start and end locations of each chunk
N=numel(Nuc);
TOTAL = N;
START = 1:chunk:TOTAL;    
END   = START + chunk-1;
END(end) = min (END(end),TOTAL);
nChunks = numel(START);
%--------------------------------------------------------

% ----------- Create variables for counting the mutants----
sumMut = zeros(size(Freq));
nmut   = zeros(size(Freq));
mTAG = cellstr (char(46*ones(size(Freq)))); % tags for mutants

% replace blanks in char array with zeros
% IX = find(cNumbers==' ');
% cNumbers(IX)='0';
%--the above script doesn't work... think..
    

    
time1 = num2str(toc,3);
disp(['done in ' time1 ' sec']);


for NN = 1:nChunks-1

    tic;    
    disp(['********* Chunk #' num2str(NN) ' out of ' num2str(nChunks) ]);
    TOTn = (END(NN)-START(NN));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Section where mutants are made %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    cNuc = char(Nuc{START(NN):END(NN)});
    N=size(cNuc,1);
    L=size(cNuc,2);

    cMUT = char ( 32*ones(N,3*L*(L+1)) ); 
    cMUT(:,end) = char(10);
    cTG = char ( 32*ones(N,3*L*(4+1)) ); 
    cTG(:,end) = char(10);

    ATCG = 'ATCG';

    for a=1:L
       col=cNuc(:,a);
       apitch = 1+(L+2)*(a-1);
       for mm=1:4
           IX=find(col==ATCG(mm));
           others = OTHER(ATCG(mm));
           for ll=1:3
               lpitch = (L+1)*L*(ll-1);
               cMUT(IX,apitch+lpitch) = others(ll);
               
               cTG(IX,1+5*L*(ll-1)+(a-1)*5+1 ) = ATCG(mm);
               cTG(IX,2+5*L*(ll-1)+(a-1)*5+1 ) = num2str(floor(a/10)); 
               cTG(IX,3+5*L*(ll-1)+(a-1)*5+1 ) = num2str(a-floor(a/10)*10);
               cTG(IX,4+5*L*(ll-1)+(a-1)*5+1 ) = others(ll);
               
               if mm==1
                   IXS = 1:(a-1);
                   IXE = (a+1):L;
                   cMUT(:,lpitch+IXS+apitch-a)=cNuc(:,IXS);
                   cMUT(:,lpitch+IXE+apitch-a)=cNuc(:,IXE);
               end
           end

       end

    end

    test2 = textscan(sprintf('%s\n', cMUT'), '%s');
    MUT = test2{1};
    test2 = textscan(sprintf('%s\n', cTG'), '%s');
    TG = test2{1};
    clear('cMUT','cTG','cNuc','ACTG','col','IXS','IXE');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% End of section where mutants are made %%%%%%%%%%%



%%%%%%%%%%%%%% Analysis Section %%%%%%%%%%%%%%%%%%%
% use ismember to compare a matrix of mutants and a matrix of sequences
% IND has order as size of MUT matrix, but elements contain addres of
% seqeunces in Nuc matrix; 0 at i-th position means i-th mutant  was not
% found. Search for mutants only in lower abundance nucleotides.
% For each chunk, the Nuc matrix is trimmed to the beggining of the chunk

   [~, IND] = ismember(MUT, Nuc (START(NN):end) );
   clear('MUT');

   % the indeces of mutants that were found is stored in fIND
   % indices of corresponding nucleotides are in IND(fIND)+START(NN)-1;
   fIND = find(IND);
   NucIX= IND(fIND)+START(NN)-1;
   NMUT = zeros(3*L,chunk);
   NMUT(fIND) = Freq ( NucIX );
   MUTix = zeros(3*L,chunk);
   MUTix(fIND)= IND(fIND)+START(NN)-1;
   
   sqTG = cell(3*L,chunk);
   sqTG(fIND) = TG(fIND);

   % create preffix for tagging the mutants
   % frequencies are calculared in a chunk matrix, hence the indices
   % must be adjusted

   for N=1:size(MUTix,2)  % repeat for all found mutants
       
       row = N +START(NN)-1;
       Rs = num2str(row);
       
       A=find(MUTix(:,N));
       for j=1:numel(A)
           tempR = MUTix(A(j),N);
           
           if row < tempR
               strN = [char(48*ones(1, 6-numel(Rs) ))  Rs ];
               mTAG{ tempR } = [mTAG{ tempR } ...
                           'S' strN  ':' sqTG{ A(j),N } '.' ];
           else  
               MUTix(A(j),N) = 0;
                NMUT(A(j),N) = 0;
           end
       end
   end
       

   % count how many mutants were found for each parent sequence
   temp=zeros(size(NMUT));
   temp( NMUT>0 ) = 1;
   nmut(START(NN):END(NN)) = (sum(temp));
   sumMut(START(NN):END(NN)) = sum(NMUT,1);

   clear('temp'); clear('IND', 'fIND', 'NMUT');

  time1 = num2str(toc,3);
  disp(['Found ' num2str(sum( nmut(START(NN):END(NN))  ))...
        ' mutants out of ' num2str(3*L*TOTn) ...
        ' possible in ' num2str(chunk) ...
        ' sequences in ' time1 ' sec']);

end % of chunk analysis
    

%------------ SAVE all the data
disp('save the results to txt file'); tic;

%---- create numered tag for each seqeunce
N=numel(Nuc);
Numbers = 1:1:N;
test2=textscan(sprintf('%d\n', Numbers), '%s');
cNumbers=char(test2{1});

% -- analyze NNK pattern of all nucleotides K=G or T
cNuc = char ( Nuc );
L=size(cNuc,2);
IX = 3:3:L;
cNucTR = cNuc(:,IX);
isNNK = char(46*ones(size(cNucTR)));
findA = find(cNucTR=='A');
isNNK (findA)='A';
findC = find(cNucTR=='C');
isNNK (findC)='C';
clear('IX','cNucTR','findC','findA', 'Nuc');
%--------------------------------------------------

B1 = fullfile(outdir, outname);
fhN = fopen(B1, 'w');

[cPep,cNuc] = nt2aacell(cNuc,1:size(cNuc,2),0,1);
cTAG = char ( mTAG );
SP   = char ( 32*ones(size(cNuc,1),5) ); % char-array of 5 spaces
RET  = char ( 10*ones(size(cNuc,1),1) ); % char-array of returns
 
% faster alternative to num2str(FreqN). 10 times faster.
    test2=textscan(sprintf('%d\n', Freq), '%s');
    cFr=char(test2{1});

    test2=textscan(sprintf('%d\n', sumMut), '%s');
    cSM=char(test2{1});
    
    test2=textscan(sprintf('%d\n', nmut), '%s');
    cNM=char(test2{1});
    
toSave = [cNumbers SP cNuc SP cPep SP cFr SP cNM SP cSM SP isNNK SP cTAG RET]; 
% create a char-array for writing

clear ('cNuc', 'cPep', 'RET','Freq', 'cTAG', 'TAG', 'sumMut',...
       'cSM');  % get rid of variables
fprintf(fhN, '%s\r\n', toSave'); % note transposition of the matrix
clear ('toSave');  % get rid of variables

disp(['******* done saving in ' num2str(toc,4) ' sec **************']);
     

end % of function

function [letters] = OTHER(nuc)
      switch nuc
          case 'A', 
            letters(1) = 'T';
            letters(2) = 'C';
            letters(3) = 'G';
          case 'T', 
            letters(1) = 'A';
            letters(2) = 'C';
            letters(3) = 'G';
          case 'C', 
            letters(1) = 'A';
            letters(2) = 'T';
            letters(3) = 'G';
          case 'G',
            letters(1) = 'A';
            letters(2) = 'T';
            letters(3) = 'C';
      end
end