function [TAG] = maplib(INPUT, direction, marker, remTAG)
% function searches for markers in the cell array of strings of nt
% sequences (INPUT variable). Marker is a structure containing:
% marker.name = 'C7C';
% marker.Sseq = 'ABCD'    -- sequence of the start marker
% marker.Sloc = [N1 M1];  -- location of the start maker
% marker.Eseq = {'EFG'}   -- sequence of the start marker
% marker.Eloc = [N2 M2];  -- location of the start maker
%
% direction is either 'F' or 'R'
% remTAG is a tag assigned to remaining sequences; 
% if PhD12 library has no markers, the remTAG could be "P12"

if iscell(INPUT)
    cINPUT = char(INPUT);
else
    cINPUT = INPUT;
end

TAG=cell(size(cINPUT,1),1);

clear INPUT;

    for i=1:numel(marker)
        
        if direction=='F'
            Sloc = marker(i).FSloc;
            Eloc = marker(i).FEloc;
            SEARCH = [marker(i).FSseq marker(i).FEseq];
        elseif direction=='R'
            Sloc = marker(i).RSloc;
            Eloc = marker(i).REloc;
            SEARCH = [marker(i).RSseq marker(i).REseq];
        end
        
        % trim the relevant areas
        if ~isempty(Sloc)
            Sin = cINPUT(:,Sloc(1):Sloc(2));
        else
            Sin = [];
        end
        
        if ~isempty(Eloc)
            Ein = cINPUT(:,Eloc(1):Eloc(2));
        else
            Ein = [];
        end
        
        % create a continous string from trimmed start and end strings
        JOIN = [Sin Ein];
        
        % find elements with forward markers
        
        RET = char(10*ones(size(JOIN,1),1));
        JOIN=[JOIN RET];
        test2=textscan(sprintf('%s\n', JOIN'), '%s');
        IX  = find( strcmp(SEARCH,test2{1}) );
        TAG(IX) = {marker(i).name};
    end

    %rem = (TAG==0);
    rem=find(cellfun('isempty', TAG));
    TAG(rem) = {remTAG};
    TAG=char(TAG);

end