function rawseq(varargin)
% Usage:


inname = '';  
chunk = 250000;     % default value
indir = '';
outdir = '';
outname= 'rawseq';   % default save name, will have 000N appended to it


% check for input variable
if exist('varargin','var')
    L = length(varargin);
    if rem(L,2) ~= 0, error('Parameters/Values must come in pairs.'); 
    end

    % read input variables
    for ni = 1:2:L
        switch lower(varargin{ni})
            case 'inname', inname = varargin{ni+1};
            case 'chunk',   chunk = varargin{ni+1};
            case 'outdir', outdir = varargin{ni+1};
            case 'outname', outname = varargin{ni+1};
            case 'indir', indir=varargin{ni+1};
            case 'searchf', searchF=varargin{ni+1};
            case 'searchr', searchR=varargin{ni+1};
            case 'barl', BARL=varargin{ni+1};
            case 'nnkl', NNKL=varargin{ni+1};
            case 'marker', marker=varargin{ni+1};
        end
    end
end

% check whether outdir name was defined
if strcmp(outdir,'')
    outdir = fullfile(indir, '_RAW');
end

if ~isdir(outdir)
    mkdir(outdir);    
end

jj=0;
for kk=1:numel(inname)
    
    FID = fopen(fullfile(indir, inname{kk}));

    START=0;
    BlockL=0;
    fseek(FID,0,'eof');
    LAST = ftell(FID);  % position of the last element of the file
    currL2 = cell(1,4);

    % read 4 lines to estimate the length of the block
    frewind(FID);
    for i=1:4 
       currL2{i} = fgets(FID);
       BlockL = BlockL + numel(currL2{i}) ;
    end
    TOT = floor(LAST/(chunk*BlockL));

    for ii=1:TOT
        tic
        [Seq, Qua, START] = loadNparse(FID,TOT,chunk,BlockL,ii,START);
        NN = num2str(size(Seq,1));
        t1 = num2str(toc,2); tic
        
        toSave = parseq(Seq, Qua,... 
                         searchF, searchR, BARL, NNKL, marker,0);
        clear('Seq', 'Qua');             
        t2 = num2str(toc,'%10.1f'); tic;
        jj=jj+1;
        currentname = makename(jj,outname);
        fh = fopen(fullfile(outdir, [currentname '.txt']), 'w');
        fprintf(fh, '%s\r\n', toSave');

        t3 = num2str(toc,'%10.1f');
        disp(['read ' inname{kk} '; ' NN ' lines; ' ...
              'wrote to ' currentname ' Load=' t1 ...
              ' Parse=' t2 ' Save=' t3]);
        fclose(fh);

    end

end % of for

fclose('all');

end

function [Seq, Qua, START] = loadNparse(FID,TOT,chunk,BlockL,step, START)

    fseek(FID,0,'eof');
    LAST = ftell(FID);  % position of the last element of the file
    
    %read lines at the end of the chunk
    if step==TOT
        END=LAST;
    else
        fseek(FID,step*(chunk-1)*BlockL,'bof');
        temp = (fread(FID, 2*BlockL, '*char'))';
        END = step*(chunk-1)*BlockL + min(strfind(temp,'@ILLUMINA'))-1;
    end

    fseek(FID,END-1,'bof');
    test = fread(FID, 1, '*char');
    if double(test) ~= 10
        disp([ 'incorrect mapping of the read; last symbol is ' test]);
    end

    fseek(FID,START,'bof'); 
    currL = (fread(FID, END-START, '*char'))';
    START=END;

    A4 = strfind(currL, char(10))';   %find end of lines
    TOT = numel(A4);
    SeqST = A4(1:4:TOT)+1;
    SeqEN = A4(2:4:TOT)-1;
    QuaST = A4(3:4:TOT)+1;
    QuaEN = A4(4:4:TOT)-1;
    L1 = unique(1+SeqEN-SeqST);
    L2 = unique(1+QuaEN-QuaST);
    
    if numel(L1) > 1 || numel(L2) > 1 || L1 ~= L2
        disp ('Problem!! reads of different lengths');
    else
        L = L1;
    end

    IX = zeros(size(SeqST,1),L);    
    for i=1:size(IX,2)
        IX(:,i) = SeqST + i-1;
    end
    Seq = currL(IX);
    
    for i=1:size(IX,2)
        IX(:,i) = QuaST + i-1;
    end
    Qua = currL(IX);   
end

function outname = makename(num, inname)
    Snum = num2str(num);
    switch numel(Snum)
        case 1
            Snum = ['000' Snum];
        case 2
            Snum = ['00' Snum];
        case 3
            Snum = ['0' Snum]; 
    end
    outname = [inname Snum];   
end

