function sortlib2(varargin)
% this function uses parsed nucleotide files and softs the entries
% according to the library type and bar type.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

inname = 'data';   % if inname stays blank, call uigetfile
outname= 'data';   % default save name, will have 000N appended to it
indir = '';
outdir = '';
CUTOFF = '.';  % default cutoff for Phred score


%check for input variable
if exist('varargin','var')
    L = length(varargin);
    if rem(L,2) ~= 0, error('Parameters/Values must come in pairs.'); 
    end

    % read input variables
    for ni = 1:2:L
        switch lower(varargin{ni})
            case 'inname', inname = varargin{ni+1};
            case 'outdir', outdir = varargin{ni+1};
            case 'outname', outname = varargin{ni+1};
            case 'indir', indir=varargin{ni+1};
            case 'cutoff', CUTOFF=varargin{ni+1};
            case 'marker', marker=varargin{ni+1};
            case 'libraries', LIBS=varargin{ni+1};
            case 'barcodes', BARF=varargin{ni+1};
            case 'limits', LIM=varargin{ni+1};
        end
    end
end

% check whether outdir name was defined
if strcmp(outdir,'')
    outdir = fullfile(indir, '_RAW');
end

if ~isdir(outdir)
    mkdir(outdir);    
end

%%%%%%%%%%%%% End of the Input Section %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FOL = cell(numel(LIBS),numel(BARF));
Fh = FOL;

fclose all;
for i=1:numel(LIBS)
    for j=1:numel(BARF)
        FOL{i,j} = fullfile(outdir, ['data122711' LIBS{i} '_' BARF{j}]);
        if ~isdir(FOL{i,j})
            mkdir(FOL{i,j});
        end

        Fh{i,j,1}=fullfile(FOL{i,j},...
            [outname 'F.txt']);
        Fh{i,j,2}=fullfile(FOL{i,j},...
            [outname 'R.txt']);
        fopen(Fh{i,j,1},'w');
        fopen(Fh{i,j,2},'w');
      
    end
end
fclose all;

%%% find all files input files and cycle through all
files = dir(fullfile(indir, [inname '*.txt']));

NN = numel(files);

% process all files
for iii=1:NN

%---create output directories and files
tic
    currentname = files(iii).name; tic
    fh = fopen(fullfile(indir, currentname), 'r+');
   
    %     LIB FR QUAL NNK BAR ADAP R36 QNNK QBAR QADAP QR36    
    % the structure of the input file
    %-read the file into a character string, re-shape it
    currL = fread(fh, '*char')';
    fclose(fh);

    RET = strfind(currL(1:10000), char(10)).';
    L = max ( unique (RET-[0; RET(1:end-1)]) );
    N = size(currL,2);
    TOT=round(N/L);
    A =  transpose(reshape (currL(1:TOT*L),L,TOT));
    clear('currL');
    timeR1 = num2str(toc,'%4.1f'); tic

    SP = strfind(A(1,:),' ');
    Qual = [ A(:,SP(9)+1:SP(10)-1)  A(:,SP(11)+1:end-1) ];
    N0=size(Qual,1);
  
    % -- ignore barcodes and nucleotides with bad quality
    [row,~] = find(Qual<CUTOFF & Qual~=' ' & Qual~='.');
    row = unique(row);
    ALL     = 1:1:N0;
    IX      = setdiff(ALL,row);
    clear('ALL','row');

    % -- extract other variables from filtered VAR-input
    LIBTag = char2cell( A(IX,1:SP(1)-1));
    FRTag  = A(IX,6);
    BARSeq = char2cell( A(IX,SP(4)+1:SP(5)-1));
    R36Seq = A(IX,SP(6)+1:SP(7)-1);
    N1=numel(BARSeq);
    clear('Qual','A','IX');

    timeR2 = num2str(toc,'%4.1f'); tic

NB=zeros(numel(LIBS),numel(BARF)+1); 

for lll=1:numel(LIBS)
    IXL = find(strcmp(LIBS(lll),LIBTag));
    NB(lll,end) = numel(IXL);
    TOT_IXB = [];
    
    for bbb=1:numel(BARF)-1
        IXB = find(strcmp(BARF(bbb),BARSeq(IXL)));
        TOT_IXB = [TOT_IXB; IXB];
        
        NB(lll,bbb) = numel(IXB);
        IX = cell(1,2);
        IX{1} = find(FRTag(IXL(IXB))=='F' );
        IX{2} = find(FRTag(IXL(IXB))=='R' );
        
        for II=1:2
            
            % find indices of seq. tagged by current LIB and BAR tags
            SaveIX = IXL( IXB(IX{II}) );

            % trim the library portion to pre-set limits
            sR36 = R36Seq(SaveIX,LIM{lll,II}(1):LIM{lll,II}(2));

            % re-save the values into the same file
            SAVE2(sR36,Fh{lll,bbb,II},II-1)
        end
    end
    
    % find barcodes with errors
    ALL = 1:1:NB(lll,end);
    DIF  = setdiff(ALL,TOT_IXB);
    NB(lll,end-1) = numel(DIF);
    IX = cell(1,2);
    IX{1} = find(FRTag(IXL(DIF))=='F' );
    IX{2} = find(FRTag(IXL(DIF))=='R' );
    
    for II=1:2
        SaveIX = IXL( DIF(IX{II}) );
        sR36 = R36Seq(SaveIX,LIM{lll,II}(1):LIM{lll,II}(2));
        sBAR = BARSeq(SaveIX);
        SAVE3(sBAR, sR36,Fh{lll,end,II},II-1)
    end
    if iii==1
        disp(BARF');
        disp([LIBS{lll} '  ' num2str(NB(lll,:)) ]);
    end
end

clear('LIBTag','BARSeq','R36Seq');

    timeW = num2str(toc, '%4.1f');
    disp([currentname ' Read:' timeR1 '+' timeR2 ' s; lost: ' ...
          num2str(N-TOT*L) ' chars; Sort/Write:' timeW  ...
          '  s; Tot Seq=' num2str(N0) ...
          ' Hi-qual(>' CUTOFF ')=' num2str(N1) ...
          ' (' num2str(round(100*N1/N0)) '%)' ]);

end  % end of for-iii-loop
end

function [CELL] = char2cell (CHAR)
    RET   =  char(10*ones(size(CHAR,1),1));
    RET(end) = ' ';
    test2 = textscan(sprintf('%s\n', [CHAR RET]'), '%s');
    CELL = test2{1};
end


function SAVE2(NUC,FILE,FR)

    RET  = char (10*ones(size(NUC,1),1));
    RET(end) = ' ';
    if FR==1
        [~,NUC]=nt2aacell(NUC,0,1,0);
    end
    toSave=[NUC RET];
    clear('NUC','RET');
    
    fh  = fopen( FILE, 'a');
    fprintf( fh, '%s\r\n', toSave');
    fclose(fh);
    clear('toSave');
end

function SAVE3(BAR,NUC,FILE,FR)

    RET  = char(10*ones(size(NUC,1),1));
    SP   = char(32*ones(size(NUC,1),5));
    sBAR = char(BAR);
    if FR==1
        [~,NUC]=nt2aacell(NUC,1:size(NUC,2),1,0);
    end
    toSave=[sBAR SP NUC RET];
    clear('NUC','RET');
    
    toSave = sortrows(toSave);
    
    toSave(end)=' ';
    
    fh  = fopen( FILE, 'a');
    fprintf( fh, '%s\r\n', toSave');
    fclose(fh);
    clear('toSave');
end