function [out] = values2deepzebra (varargin)
% by definition the input file has the following structure
% nucleotide peptide frequency other
% nucleotide peptide frequency other
% ...        ...     ...       ...
% if it has a different structure you have to change the
% inputform and readfields variables


inputformat = '%*s %*s %d %*[^\n]';
readfields = 1;

% if the file hasn't been frequency-analyzed, and contains repetitive
% sequences, it has to be analyzed by other script first
%%%%%%%%%%%%%%%%%%%%

findunique=1;
H=100;
color = 'jet';
sublength=0;
zebra = defaultzebra;
zebra.MIRROR = 0;

% by definition the Fzebra bar zebra.segments contain the following frequencies
zebra.segment=[1 3 6 10 30 100 300 1000 3000 10000 30000 100000 ...
          300000 1000000 3000000];  
      
if exist('varargin','var')
    L = numel(varargin);
    if rem(L,2) ~= 0, error('Parameters/Values must come in pairs.'); 
    end

    % read input variables
    for ni = 1:2:L
        switch lower(varargin{ni})

            case 'fr', Fr = varargin{ni+1};
            case 'nuc', Nuc = varargin{ni+1};
                
            case 'color', color = varargin{ni+1};
            case 'figurehandle', H = varargin{ni+1};    
            case 'findunique', findunique = varargin{ni+1}; 
            case 'sublength', sublength = varargin{ni+1};
            case 'xy00', XY00 = varargin{ni+1};
            case 'normalized', NORMAL = varargin{ni+1};
            
            case 'dimension', zebra.dimension = varargin{ni+1};    
            case 'xlim', zebra.xLim = varargin{ni+1}; 
            case 'ylim', zebra.yLim = varargin{ni+1};                
            case 'width', zebra.width = varargin{ni+1}; 
            case 'lenght', zebra.length = varargin{ni+1}; 
            case 'addtotal', zebra.addtotal = varargin{ni+1}; 
            case 'addlabel', zebra.addlabel = varargin{ni+1};
            case 'addfreq', zebra.addfreq = varargin{ni+1};    
            case 'addaxis', zebra.addaxis = varargin{ni+1};  
            case 'textheight', zebra.Th = varargin{ni+1};
            case 'font', zebra.Font = varargin{ni+1};
            case 'title', zebra.Title = varargin{ni+1};
            case 'orientation', zebra.orientation = varargin{ni+1};
            case 'direction', zebra.direction = varargin{ni+1}; 
            case 'mirror', zebra.MIRROR = varargin{ni+1};
            case 'figurehandle', H = varargin{ni+1}; 
                
            case 'segment', zebra.segment = varargin{ni+1};
            case 'nnkonly', NNKonly = varargin{ni+1};
        end
    end

end

% if only NNK sequences are considered, discard the non-NNK sequences

if NNKonly
    IX  = isNNK(Nuc);
    Nuc = Nuc(IX);
    Fr  = Fr(IX);
end

% if unique sequences and frequencies need to be found, do it here:    
if findunique
    
    [uNuc, Fr, ~] = uniqueCOMB (Nuc, Fr); 
else
    uNuc = AllVar{readfields(1)};
end
    SUM=sum(Fr);
    clear('AllVar'); fclose all;
% break peptides into sub-peptide motifs  
    if sublength
       cuNuc=char(uNuc);
       L=size(cuNuc,2);
       subNuc = [];
       subFr  = [];
       for i=1:(L-sublength+1)
           subNuc=[subNuc; cuNuc(:,i:i+sublength-1)];
           subFr =[subFr;  Fr];
       end
       [~, Fr, ~] = uniqueCOMB (cellstr(subNuc), subFr); 
       clear('cuNuc', 'subNuc', 'subFr');
    end


% blank the variables for the Fzebra bar
zebra.W=zeros(size(zebra.segment));
zebra.H=zeros(size(zebra.segment));
zebra.MaxA = 0;
zebra.Uni=0;
zebra.Tot=sum(Fr);  % blank the counters for unique and total sequences


%%%% normalize the frequencines 
if NORMAL
    SINGLETON = 1 / sum(Fr);
    Fr = Fr / sum(Fr); 
    SUM = 1;
    zebra.segment(1) = SINGLETON;
    zebra.Nseg = numel(zebra.segment);
else
    SINGLETON = 1;
    zebra.Nseg = find(zebra.segment>max(Fr), 1 );
end

% predict number of zebra.segments
   
for i=1:zebra.Nseg  % calculate the width and heigh of zebra.segments
    
    if i==1
        IX=find(Fr==SINGLETON); 
    else
        IX = find(Fr>zebra.segment(i-1) & Fr<=zebra.segment(i) & ...
                  Fr>zebra.segment(1));
    end

    zebra.W(i) = numel(Fr(IX));          % find the width of zebra.segments
    zebra.segfreq(i) = sum(Fr(IX));      % number of seq. per segment     
    zebra.Uni = zebra.Uni + zebra.W(i);  % add to unique number of seq.
    zebra.H(i) = sum(Fr(IX))/SUM;        % find the height of zebra.segment
    zebra.label{i} = num2str(zebra.segment(i));
    
    if strcmpi(zebra.dimension,'1d')
        zebra.addaxis = 0;
        zebra.drawoutline=1;
        zebra.W(i) = 1;                      % all widths are equal
    elseif strcmpi(zebra.dimension,'2d')
        zebra.addaxis = 1;
        zebra.drawoutline=0;
        zebra.W(i) = (log10(zebra.W(i))+1);  % convert width to log-scale
        
        if (log10(numel(Fr(IX)))+1)>zebra.MaxA
            zebra.MaxA = (log10(numel(Fr(IX)))+1);
        end
    end
    
end  % of generation of zebra.segments for individual zebra-bar 

%%%%%%%%%%%%%%%%%%  pick a color scheme (jet by default) %%%%%%%%%%    
zebra = ColorZebra(zebra, color);

%%%%%%%%%% define the drawing window and limits within the window    
figure(H);
xlim(zebra.xLim); ylim(zebra.yLim);

%%%%%%%%%%%%%%%%%%   draw the zebra.segments   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    zebra.XY00 = XY00;  
    if ~zebra.MIRROR
        struct2zebra(zebra,0);
    else
        struct2zebra(zebra,1);
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

out = zebra;
 
end % of function

function [rows, outlogical] =  isNNK(inchar)

if iscell(inchar)
    inchar = char(inchar);
end
[N,M]=size(inchar);

m=floor(M/3);
temp=ones(N,m);
for i=1:m
    temp(:,i)= (inchar(:,i*3) == 'A' | inchar(:,i*3) == 'C');
end

outlogical=not(any(temp,2));
rows = find(outlogical);
end

function [out] = defaultzebra(varargin)

%  zebra bar is horizontal and it has bottom left corner at 0,0
zebra.xLim = [-0.5 1.5]; 
zebra.yLim = [-1 2];
zebra.orientation ='horizontal';
zebra.direction   ='ascend';


% by default there are no labels and no axis
zebra.addtotal=0;
zebra.addlabel=0;
zebra.addfreq=0;
zebra.drawoutline=1;
zebra.dimension='1D';
zebra.addaxis=0;
zebra.Font = 'Monospaced';

% default size and text fonts sizes
zebra.width=0.3; 
zebra.length = 1;  
zebra.Th=0.1; 
zebra.Uni=0;
zebra.Tot=0;  % restart the counters for unique and total sequence


out = zebra;    
    
end

function [zebra] = ColorZebra(zebra, color)
% this function works only with jet color scheme

    if isfield(zebra,'segment');
        temp = zebra.Nseg;
        zebra.Nseg = max(zebra.Nseg,numel(zebra.segment));
    end

    zebra.COL={};
    zebra.COL2={};

    if strcmp(color,'jet')
      N=ceil(size(jet,1)/zebra.Nseg);
      A=jet(zebra.Nseg);
      for i=1:zebra.Nseg
          zebra.COL{i}=A(i,:);
          zebra.COL2{i} = [0 0 0];
      end
    end
    
    if isfield(zebra,'segment');
        zebra.Nseg = temp;
    end
end
