function values2scatter(varargin)
% example of usage
%
% values2scatter('Xfreq', H.Fr{1},'Yfreq', H.Fr{2},...
%                   'Xseq', H.Nuc{1},'Yseq', H.Nuc{2},...
%            'outname', '', 'normalizedX',1,'normalizedY',1,...
%            'resize',1,...
%            'inputformat',inputstring,...
%            'readlines', [1 2],...
%            'namex', 'XaxisName', 'namey', 'XaxisName');

outdir = '';
outname = '';

namex='';
namey='';
reunique = 1;
MAXX=0;
MAXY=0;
resize=1;
normalizedx=0;
normalizedy=0;
NNKonly=0;
plotratio=1;

if exist('varargin','var')
    L = length(varargin);
    if rem(L,2) ~= 0, error('Parameters/Values must come in pairs.'); 
    end

    % read input variables
    for ni = 1:2:L
        switch lower(varargin{ni})
            case 'xfreq',       xfreq = varargin{ni+1};
            case 'yfreq',       yfreq = varargin{ni+1};
            case 'xseq',        xseq = varargin{ni+1};
            case 'yseq',        yseq = varargin{ni+1};    
            
            case 'outdir',      outdir=varargin{ni+1};         
            case 'outname',     outname = varargin{ni+1};
            case 'figurehandle', hh = varargin{ni+1};

            case 'namex',       namex = varargin{ni+1};
            case 'namey',       namey = varargin{ni+1};
            case 'reunique',    reunique = varargin{ni+1};
            case 'maxx',        MAXX = varargin{ni+1};               
            case 'maxy',        MAXY = varargin{ni+1};
            case 'normalizedx', normalizedx = varargin{ni+1};
            case 'normalizedy', normalizedy = varargin{ni+1}; 
            case 'resize',      resize = varargin{ni+1}; 
            case 'nnkonly',     NNKonly = varargin{ni+1};  
                
            case 'flipxy',      flipXY = varargin{ni+1};
            case 'plotratio',   plotratio = varargin{ni+1};
                
            case 'symaxis', symmetric_axis = varargin{ni+1};
        end
    end
end

if flipXY
    tempseq = xseq;
    tempfre = xfreq;
    xseq    = yseq;
    xfreq   = yfreq;
    yseq    = tempseq;
    yfreq   = tempfre;
    clear('tempseq','tempfre');
end

if NNKonly
    IX      = isNNK(xseq);
    xseq    = xseq(IX);
    xfreq   = xfreq(IX);
    
    IX      = isNNK(yseq);
    xseq    = yseq(IX);
    yfreq   = yfreq(IX);
    clear IX;
end

if ~strcmp(outdir, '')    
    if ~isdir(outdir)

        mkdir(outdir);
    end
end

%%%%%%%%% normalize the sequences
if normalizedx
    xfreq = xfreq / sum(xfreq);
end
if normalizedy
    yfreq = yfreq / sum(yfreq);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if the sequences have not been processed to find unique, do it here

if reunique
    [xseq, xfreq, ~] = uniqueCOMB (xseq, xfreq); 
    [yseq, yfreq, ~] = uniqueCOMB (yseq, yfreq);
end

[~, iX1, iY1] = intersect(xseq, yseq); % finds all common seq in both files

CommonX1Seq = xseq(iX1); % makes a list of all common seq from X1 and Y1 
CommonY1Seq = yseq(iY1); % frequency of common sequences in Y

if plotratio
    CommonX1Nu = xfreq(iX1); % frequency of common sequences in X
    CommonY1Nu = yfreq(iY1)./xfreq(iX1); % fold increase of Y over X
else
    CommonX1Nu = xfreq(iX1); % frequency of common sequences in X
    CommonY1Nu = yfreq(iY1); % frequency of common sequences in Y
end
 

clear('iX1','iY1');

% end of finding common items and extracting numbers for them
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%% Find min and max of the plot

if MAXX==0 || MAXY==0
    MAXX = 10^(ceil(log10 ( double(max(xfreq)) ) ));
    if plotratio
        MAXY = 10^(ceil(log10 ( double(max(CommonY1Nu)) ) ));
    else
        MAXY = 10^(ceil(log10 ( double(max(yfreq)) ) ));
    end
end
% MINX = 10^(floor(log10 ( double(min(CommonX1Nu)) ) ));
% MINY = 10^(floor(log10 ( double(min(CommonY1Nu)) ) ));

MINX = min(xfreq);
if plotratio
    MINY = min(CommonY1Nu);
else
    MINY = min(yfreq);
end

if plotratio
    gridX =10.^(log10(MINX):1:log10(MAXX));
    gridY =10.^(floor(log10(MINY)):1:ceil(log10(MAXY)));
else
    if ~symmetric_axis
        gridX =10.^(log10(MINX):1:log10(MAXX));
        gridY =10.^(log10(MINY):1:log10(MAXY));
    else
        MAXXY=max(MAXX,MAXY);
        gridX =10.^(log10(MINX):1:log10(MAXXY));
        gridY =10.^(log10(MINY):1:log10(MAXXY));
    end
end

% Find sequences that are NOT commont between two sets

[~, iA2] = setdiff(xseq, yseq); % finds diffeneces in X1 that are not in Y1
difX1Nu = xfreq(iA2);           % lists the corresponding frequencies
blankdifX1Nu = 0.3*MINY*ones( numel(difX1Nu), 1);
clear('iA2');

[~, iA2] = setdiff(yseq, xseq);
difY1Nu = yfreq(iA2); 
blankdifY1Nu = 0.3*MINX*ones( numel(difY1Nu), 1);
clear('iA2');
    


%%%%% plot the sequences 
figure(hh);
freqs = [3 10 30 100 300 1000 3000 10000 30000];
%freqs = [1 3 10 100 1000 10000 100000];
hold on


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plotgrid(gridX,gridY);
colorplot(difX1Nu,blankdifX1Nu,freqs,[1 0 0],1,resize);

if ~plotratio
    colorplot(blankdifY1Nu,difY1Nu,freqs,[0 0 1],2,resize);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

colorplot(CommonX1Nu, CommonY1Nu,freqs,[0 1 0],1,resize);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    MAX = max(MAXX, MAXY);
    MIN = min(MINX, MINY);
      
if ~plotratio
    % plot y=x straight line
    h=line([MIN MAX], [MIN MAX], 'color', [0.5 0.5 0.5]); 
    if ~isempty(CommonX1Nu)
        text(MAXX, MAXY, num2str(numel(CommonX1Nu)) ,...
             'rotation',0, 'color', [0 0 0]);
    end
else
    % plot y=1 straight line
    h=line([MIN MAX], [1 1], 'color', [0.5 0.5 0.5]);
    if ~isempty(CommonX1Nu)
        text(MAXX, 1, num2str(numel(CommonX1Nu)) ,...
             'rotation',0, 'color', [0 0 0]);
    end  
end
    
    xticks = 10.^(floor(log10(MINX)):1:ceil(log10(MAXX)));
    yticks = 10.^(floor(log10(MINY)):1:ceil(log10(MAXY)));
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

if plotratio
    OUT = countINgrid([CommonX1Nu; difX1Nu],...
                      [CommonY1Nu; blankdifX1Nu],...
                       gridX,gridY);
else
    OUT = countINgrid([CommonX1Nu; difX1Nu; blankdifY1Nu],...
                      [CommonY1Nu; blankdifX1Nu; difY1Nu],...
                       gridX,gridY);
end
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Andjust the axis to log-scale; draw thik-marks

set(get(h,'Parent'), 'YScale', 'log', 'XScale', 'log',...
    'TickDir', 'out','XMinorTick','on', 'YMinorTick','on',...
    'XTick',xticks, 'YTick', yticks);


if ~symmetric_axis 
    xlim([0.1*MINX MAXX]);
    ylim([0.1*MINY MAXY]);
else
    MAXXY = max(MAX,MAXY);
    xlim([0.1*MINX MAXXY]);
    ylim([0.1*MINY MAXXY]);
end
    

if ~isempty(blankdifX1Nu) && ~isempty(difX1Nu)
    text(1.2*double(max(difX1Nu)), double(blankdifX1Nu(1)),...
         num2str(numel(blankdifX1Nu)),...
         'rotation',0, 'color','r',...
         'VerticalAlignment','middle');
end

if ~isempty(blankdifY1Nu) && ~isempty(difY1Nu) && ~plotratio
    text(double(blankdifY1Nu(1)), 1.2*double(max(difY1Nu)),...
         num2str(numel(blankdifY1Nu)),...
         'rotation',90, 'color','b',...
         'VerticalAlignment','middle');
end
 
hold off

if ~strcmp([namex namey],'')
    xlabel(['Sequence in ' namex], 'fontsize',12)
    ylabel(['Sequence in ' namey], 'fontsize',12)
    title([namex ' vs ' namey],...
          'fontsize',12, 'fontweight','b')
end


if ~strcmp([outdir outname], '')
    print(hh, '-djpeg', [outdir outname '.jpg']);
    print(hh, '-depsc', [outdir outname '.eps']);
end
fclose('all');
    
end

function colorplot(x,y,freqs,COLOR, flip, resize)

% if datasets are empty, quit the function
if isempty(x) || isempty(y)
    return
end
% this algorithm works only when values are >1
% if they are <1, they have to be scaled first

if min(x)<1
    scalex = 100*10^( -floor(log10(min(x))) );
    x = round(scalex*x);
else
    scalex=1;
end

if min(y)<1
    scaley = 100*10^( -floor(log10(min(y))) );
    y = round(scaley*y);
else
    scaley = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% in this section we count number of occurences of each coordinate

    MAX = max([max(x) max(y)]);
    COEF = 10^( ceil(log10(MAX))+1 );
    if flip==1    
    z = x*COEF + y;                % collapse two coordinates in one number
    z = sort(z);
    [sZ,M,~] = unique(z, 'first'); % finds unique z1
    M2 = [M(2:end); numel(z)+1];   % creates shifted index array M2
    Occur = M2-M;                  % find the number of occurences
    x1 = floor(sZ/COEF);           % transform back to xy-coordinates
    y1 = (sZ-x1*COEF);
    elseif flip==2
    z = y*COEF + x;                % collapse two coordinates in one number
    z = sort(z);
    [sZ,M,~] = unique(z, 'first'); % finds unique z1
    M2 = [M(2:end); numel(z)+1];   % creates shifted index array M2
    Occur = M2-M;                  % find the number of occurences
    y1 = floor(sZ/COEF);           % transform back to xy-coordinates
    x1 = (sZ-y1*COEF);
    end
        

    [Occur, IX ] = sort(Occur);    % sort occurences and coordinates
    maxOccur = max(Occur);
    x1=x1(IX);
    y1=y1(IX);

    IND(1) = 1;
    ii=1;
    for i=1:numel(freqs)           % find indeces for each frequency range
        ii=ii+1;
        if ~isempty( find(Occur>=freqs(i), 1  ) )
            IND(ii) = find( Occur>=freqs(i) , 1 ) ;
        else
            IND(ii) = NaN;
        end
    end
    clear ii;

    IND(numel(freqs)+1) = numel(Occur)+1;
    IND2 =  IND (~isnan(IND));
    FRA = 1/numel(freqs);

    for i = 1:numel(IND2)-1

        PLOT = IND2(i):IND2(i+1)-1;
        hold on;
        if resize
            SIZE = 1+i;
        else
            SIZE = 2;
        end

        plot(x1(PLOT)/scalex,y1(PLOT)/scaley,...
                        'o',...
                        'MarkerEdgeColor',(1-FRA*i)*COLOR,...
                        'MarkerFaceColor',(1-FRA*i)*COLOR,...
                        'MarkerSize',SIZE);  
    end
end

function plotgrid(gridX,gridY)

    gridX = [0.4*min(gridX) gridX];
    gridY = [0.4*min(gridY) gridY];

    for i=1:numel(gridX)-1
         line(gridX(i)*[1 1],[0.1*min(gridY) gridY(end)],...
             'color',[0.7 0.7 0.7]);
    end

    for i=1:numel(gridY)-1
        line([0.1*min(gridX) gridX(end)], gridY(i)*[1 1],...
            'color',[0.7 0.7 0.7]);
    end
end

function [OUT] = countINgrid(X,Y,gridX,gridY)
 
 gridX = [0.1*min(gridX) 0.4*min(gridX) gridX];
 gridY = [0.1*min(gridY) 0.4*min(gridY) gridY];
 
 OUT = zeros(numel(gridX),numel(gridY));
 
 for i=1:numel(gridX)-1

     for j=1:numel(gridY)-1
         % count number of data points in each cell of the grid
         XIND = find(X>gridX(i) & X<=gridX(i+1));
         Yt= Y(XIND);
         N = numel(find(Yt>gridY(j) & Yt<=gridY(j+1)));
         OUT(i,j) = N;
         if N>0
         text(10^((log10(gridX(i))+log10(gridX(i+1)))/2),...
              10^((log10(gridY(j))+log10(gridY(j+1)))/2),...
              num2str(N),...
             'HorizontalAlignment','center',...
             'VerticalAlignment','middle');
         end
         % draw a horizontal line

     end
 end

end


function [uSeqLo, sumNum, Occur] = uniqueCOMB (SeqLo, NumLo)

[SeqLo,IX] = sort(SeqLo);  
NumLo     = NumLo(IX);   % sort both arrays
%clear IX;

[uSeqLo, M, ~] = unique (SeqLo, 'first');  
M2 = [M(2:end); numel(SeqLo)+1] ;          % M2 is the last element +1
Occur = M2-M; 
%clear M2;

% for truncated seqeunces that occured only once, 
% the frequencies are the same as those of the original seqeunces

I1 = find(Occur==1); % indices of sequences that occured once
% M(Il) are the indices of the sequences that occured only once in the
% original array SeqLo
% I1 are the indices of the sequences that occured once in the unique array

sumNum = nan(size(uSeqLo));
sumNum( I1 ) = NumLo( M(I1) );
%clear I1;

% if truncations were found more than once, the frequencies must be added

I2 = find(Occur>1);  
Nu = Occur(I2);   % ths number of times each sub-seqeunce was found

for iii = 1:numel(I2)
    temp = M(I2(iii));
    sumNum( I2(iii) ) = sum (NumLo( temp:temp+Nu(iii)-1 ) );
end

[sumNum, IX] = sort(sumNum, 'descend');
 uSeqLo = uSeqLo(IX);
 Occur  =  Occur(IX);

end

function [outstring] =  isNKK(inchar)

I=[1 2 3 4 5 6 7 8 9 10 11 12]*3;
[N,M]=size(inchar);

m=floor(M/3);
temp=char(ones(N,m))
for i=1:m
    temp;
end

outstring=zeros(N,m);
outstring( instring(I(1:n)) == 'A' )=1;
outstring( instring(I(1:n)) == 'C' )=1;
end
