function [out] = values2venndia(varargin)
% by definition the input file has the following structure
% nucleotide peptide frequency other
% nucleotide peptide frequency other
% ...        ...     ...       ...
% if it has a different structure you have to change the
% inputform and readfields variables



reunique=0;
sublength=0;
NNKonly=0;
H=300;
 
      
if exist('varargin','var')
    L = numel(varargin);
    if rem(L,2) ~= 0, error('Parameters/Values must come in pairs.'); 
    end

    % read input variables
    for ni = 1:2:L
        switch lower(varargin{ni})

            case 'xfreq',       xfreq = varargin{ni+1};
            case 'yfreq',       yfreq = varargin{ni+1};
            case 'xseq',        xseq = varargin{ni+1};
            case 'yseq',        yseq = varargin{ni+1}; 
                
            case 'color',       color = varargin{ni+1};
            case 'figurehandle', H = varargin{ni+1};    
            case 'findunique',  reunique = varargin{ni+1}; 
            case 'sublength',   sublength = varargin{ni+1};
            
            case 'dimension',   zebra.dimension = varargin{ni+1};    
            case 'xlim',        zebra.xLim = varargin{ni+1}; 
            case 'ylim',        zebra.yLim = varargin{ni+1};                
            case 'width',       zebra.width = varargin{ni+1}; 
            case 'lenght',      zebra.length = varargin{ni+1}; 
            case 'addtotal',    zebra.addtotal = varargin{ni+1}; 
            case 'addlabel',    zebra.addlabel = varargin{ni+1};
            case 'addfreq',     zebra.addfreq = varargin{ni+1};    
            case 'addaxis',     zebra.addaxis = varargin{ni+1};  
            case 'textheight',  zebra.Th = varargin{ni+1};
            case 'font',        zebra.Font = varargin{ni+1};
            case 'title',       zebra.Title = varargin{ni+1};
            case 'orientation', zebra.orientation = varargin{ni+1};
            case 'direction', 	zebra.direction = varargin{ni+1}; 
            case 'mirror',      zebra.MIRROR = varargin{ni+1};
                
            case 'segment',     zebra.segment = varargin{ni+1};
            case 'nnkonly',     NNKonly = varargin{ni+1};
        end
    end

end


screen_size=get(0,'ScreenSize');
scr_width=screen_size(3);
scr_height=screen_size(4);

figure(H);
set(H,'Position',[scr_width*0.6 scr_height*0.9 ...
                  scr_width*0.35 scr_height*0.8],...
                'NumberTitle','off',...
                'Name','Venn Diagrams');
        
%if only NNK sequences are considered, discard the non-NNK sequences

if NNKonly
    IX      = isNNK(xseq);
    xseq    = xseq(IX);
    xfreq   = xfreq(IX);
    
    IX      = isNNK(yseq);
    xseq    = yseq(IX);
    yfreq   = yfreq(IX);
    clear IX;
end

% if unique sequences and frequencies need to be found, do it here:    
if reunique
    [xseq, xfreq, ~] = uniqueCOMB (xseq, xfreq); 
    [yseq, yfreq, ~] = uniqueCOMB (yseq, yfreq);
end

% % break peptides into sub-peptide motifs  
%     if sublength
%        cuNuc=char(uNuc);
%        L=size(cuNuc,2);
%        subNuc = [];
%        subFr  = [];
%        for i=1:(L-sublength+1)
%            subNuc=[subNuc; cuNuc(:,i:i+sublength-1)];
%            subFr =[subFr;  Fr];
%        end
%        [~, Fr, ~] = uniqueCOMB (cellstr(subNuc), subFr); 
%        clear('cuNuc', 'subNuc', 'subFr');
%     end

[~, iX1, iY1] = intersect(xseq, yseq); % finds all common seq in both files

CommonX1Nu = xfreq(iX1); % frequencies of the common seq in xfreq
CommonY1Nu = yfreq(iY1); % frequencies of the common seq in yfreq

clear('iX1','iY1');

% Find sequences that are NOT commont between two sets

[~, iA2] = setdiff(xseq, yseq); % finds diffeneces in X1 that are not in Y1
difX1Nu = xfreq(iA2);           % lists the corresponding frequencies
clear('iA2');

[~, iA2] = setdiff(yseq, xseq);
difY1Nu = yfreq(iA2); 
clear('iA2');
    
%%%%%%%%%%%% End of unique / difference proressing %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%% plot the unique set Venn diagram %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Xn = numel(xfreq);
Yn = numel(yfreq);
YuX = numel(CommonX1Nu);

num2venn(2.5,-2,Xn,Yn,YuX);


%%%%%%%%%%%% plot the multisest Venn diagram %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Xn = sum(xfreq);
Yn = sum(yfreq);
YuX = sum(min([CommonX1Nu CommonY1Nu], [], 2));

num2venn(2.5,1,Xn,Yn,YuX);

%%%%%%%%%%%% plot the mVenn diagram %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Xn = sum(xfreq);
Yn = sum(yfreq);
XuY = sum(CommonX1Nu);
YuX = sum(CommonY1Nu);

STRING = {['X = ' num2str(Xn) char(10) ...
           'X_m\cap Y=' num2str(XuY) ...
           ' (' num2str(100*XuY/Xn,'%5.3f')  '%)' char(10) ...
           'X_m/ Y=' num2str(Xn-XuY) ...
           ' (' num2str(100*(Xn-XuY)/Xn,'%5.3f')  '%)'];
          ['Y = ' num2str(Yn) char(10)  ...
           'Y_m\cap X=' num2str(YuX)  ...
           ' (' num2str(100*YuX/Yn,'%5.3f')  '%)' char(10) ...
           'Y_m/ X=' num2str(Yn-YuX) ...
           ' (' num2str(100*(Yn-YuX)/Yn,'%5.3f')  '%)']};
      
R  = [sqrt(Xn/pi)  sqrt(Yn/pi) ...
      sqrt(XuY/pi) sqrt(YuX/pi)];
R  = R /max(R);    %normalize the radius

X0 = 2.5;
Y0 = 3.5;

centerX = [X0-R(1) X0+R(2) X0-R(3) X0+R(4)]; %the last Y has to be calculated from YuX
centerY = ones(1,4)*Y0; 
cornerX = centerX-R;
cornerY = centerY-R;
W=[1 1 1];
COLOR   = {0.2*W 0.2*W 0.5*W 0.5*W };


for i=1:numel(cornerX)
rectangle('position', [cornerX(i) cornerY(i) 2*R(i) 2*R(i)],...
          'curvature', [1 1],'FaceColor',COLOR{i});
end


text(X0-0.1, centerY(1)+1, STRING{1},...
     'Interpreter','tex',...
      'VerticalAlignment','bottom','HorizontalAlignment','right');
text(X0+0.1, centerY(2)+1, STRING{2},...
     'Interpreter','tex',...
      'VerticalAlignment','bottom','HorizontalAlignment','left');
  
end

function num2venn(X0,Y0,Xn,Yn,YuX)

STRING = {['X = ' num2str(Xn) char(10)...
           'X / Y=' num2str(Xn-YuX) ...
           ' (' num2str(100*(Xn-YuX)/Xn,'%5.3f')  '%)'   ];
          ['Y = ' num2str(Yn) char(10)...
           'Y / X=' num2str(Yn-YuX) ...
          ' (' num2str(100*(Yn-YuX)/Yn,'%5.3f')  '%)'   ]};

      
R = [sqrt(Xn/pi) sqrt(Yn/pi)];
R = R/max(R);    %normalize the radius

% find the large circle and small circle. 
R1=max(R);
r1=min(R);

% Calculate ovelar area as fraction of the small circle area
A1=YuX/min([Xn Yn]);

% find the distance d between the circles
D=R1+r1:-0.01*r1:R1-r1;

for i=1:numel(D)
    d=D(i);
    a = (-d+r1+R1)*(d+r1-R1)*(d-r1+R1)*(d+r1+R1);

    IN1 = (d^2+r1^2-R1^2)/(2*d*r1);
    IN2 = (d^2+R1^2-r1^2)/(2*d*R1);

    A = (r1^2*acos(IN1) + R1^2*acos(IN2) - 0.5*sqrt(a)) / (pi*r1^2);
    %disp([num2str(d) '   ' num2str(A)]);
    if A>A1
        %disp('satisfied');
        break
    end
end

% place the left circle at X0 Y0 and the right circle at X0+d, Y0

centerX = [X0-0.5*d X0+0.5*d]; 
centerY = ones(1,2)*Y0; 

cornerX = centerX-R;
cornerY = centerY-R;


for i=1:numel(cornerX)
rectangle('position', [cornerX(i) cornerY(i) 2*R(i) 2*R(i)],...
          'curvature', [1 1]) 
end
text(centerX(1), centerY(1)-1, STRING{1},...
     'VerticalAlignment','top','HorizontalAlignment','right'); 
text(centerX(2), centerY(2)-1, STRING{2},...
     'VerticalAlignment','top','HorizontalAlignment','left');  

text(X0, Y0+max(R), ['X\cap Y=' num2str(YuX)],...
     'VerticalAlignment','bottom','HorizontalAlignment','center');

xlim([0 5]);
ylim([-3 6]);
axis off;

end